clear;close all;
addpath ../matlab_octave;

%===============================================================================
%
% GENERATION OF A BEAM (DISK CROSS-SECTOIN) WITH A CUBIC INCLUSION
%
%               BEAM AXIS     : 1
%
%               MATERIAL ID 0 (numM=1): additional void
%               MATERIAL ID 1 (numM=2): beam
%               ZONE ID     0 (numZ=1): void+beam/matrix
%               ZONE ID     1 (numZ=2): inclusion
%
% test octave : ok
% test matlab : ok
%===============================================================================

%                                                                         INPUTS
%===============================================================================

nx0 = 60 ;              % grid size multiple of 2 (for exact discretization of the cube)

%-------------------------------------------------------------------------------

nx = nx0 + 4;           % enlarged grid size

DX0 = 100.;             % grid dimension (assumed cubic)
dx = DX0/nx0;
DX = DX0 + 4*dx;        % enlarged grid dimension

R0 = 50.;               % radius of cylinder
r0 = 25.;               % cube radius


Rvoxcomp = dx * (sqrt(2)/2);   % Disk radius around a composite voxel
a_detect = 0.9345;             % reduction of Rvoxcomp for the DETECTION of composite voxels 
                               % -> avoid too low volume fractions
                               % 0.9345 corresponds to a threshhold of 1%

fv_threshold = (acos(a_detect)-a_detect*sqrt(1-a_detect^2))/pi

filenameM    = strcat("Cube",num2str(nx),"_cylinder_mate.vtk");
filenameZ    = strcat("Cube",num2str(nx),"_cylinder_zone.vtk");
Rep_composite= strcat("Cube",num2str(nx),"_cylinder_composite");

%                                        MICROSTRUCTURE WITHOUT COMPOSITE VOXELS
%                                               => MATE ID, ZONE ID => vtk files
%===============================================================================

% GENERATE GRID COORDINATES
%--------------------------
P0 = [0,0,0];            % grid position (lower unit-cell corner)
D0 = [DX0,DX,DX];        % grid dimension
N0 = [nx0,nx,nx];        % grid resolution (number of voxel / direction)

[X,Y,Z]=gen_grid(P0,D0,N0);


% GENERATE MATID : 0 = mat_void_matrix, 1 = mat_elastic cylinder
%---------------------------------------------------------------
P0=[0.5*DX0,0.5*DX,0.5*DX];
D0=[1,0,0];
MATEID = gen_cyl(P0,D0,R0,X,Y,Z);

savefieldvtk(MATEID,dx,dx,dx,filenameM,'materialID','int8');


% GENERATE ZONEID : mat_void_matrix : 0, mat_elastic : matrix = 0, cube=1
%            -> 0 eveywhere except 1 in the cube
%---------------------------------------------------------------------
id0=1;
P0=[0.5*DX0,0.5*DX,0.5*DX];

ZONEID = 0*MATEID;

i0 = find(  (X-P0(1)) > -r0 & (X-P0(1)) < r0 ...
          & (Y-P0(2)) > -r0 & (Y-P0(2)) < r0 ...
          & (Z-P0(3)) > -r0 & (Z-P0(3)) < r0);

ZONEID(i0) = 1;

savefieldvtk(ZONEID,dx,dx,dx,filenameZ,'zoneID','int8');


%                                             COMPOSITE VOXELS FOR BEAM CYLINDER
%===============================================================================

%     IDENTIFY AND CHARACTERIZE COMPOSITE VOXELS
%-----------------------------------------------

XC=[0.5*DX0,0.5*DX,0.5*DX];

% Distance to axis 
dist = sqrt((Y-XC(2)).^2+(Z-XC(3)).^2);

% Composite voxels identification -> k0
i0 = find(dist<=R0);                                    %i0 sphere/cylindre
j0 = find(dist>R0);                                     %j0 matrix
k0=find(dist>=(R0-a_detect*Rvoxcomp) & dist<=(R0+a_detect*Rvoxcomp));     %k0 : Voxel composite 

% Volume fraction of 'beam' within composite voxels -> fv
d1=sqrt((Y(k0)-XC(2)).^2+(Z(k0)-XC(3)).^2);
di1=(R0^2+d1.^2-Rvoxcomp^2)./(2.*d1); %distance sphere interface
S1 = R0^2*acos(di1/R0) - di1.*(R0^2-di1.^2).^0.5 ...
   + Rvoxcomp^2*acos((d1-di1)/Rvoxcomp) - (d1-di1).*(Rvoxcomp.^2-(d1-di1).^2).^0.5;
fv = S1./(pi*Rvoxcomp^2);

% verifications 
l0 = setdiff(i0,k0);
Vcyl = size(l0,1)*dx^3 + sum(fv)*dx^3;
Vcyl_i0 = size(i0,1)*dx^3;
Vcyl0 = DX * pi * R0^2;

display("error without composite voxels")
abs(Vcyl_i0 - Vcyl0) / Vcyl0

display("error with composite voxels")
abs(Vcyl - Vcyl0) / Vcyl0

display(strcat("fvmin = ",num2str(min(fv))));
display(strcat("1-fvmax = ",num2str(1-max(fv))));

% Interface normal vector -> Nx,Ny,Nz
Nx=X(k0)*0;
Ny=Y(k0)-XC(2);
Nz=Z(k0)-XC(3);
NormeN=sqrt(Nx.^2+Ny.^2+Nz.^2);
Nx=Nx./NormeN;
Ny=Ny./NormeN;
Nz=Nz./NormeN;

% Interface tangent vector -> Tx,Ty,Tz
Vx=ones(size(k0));
Vy=zeros(size(k0));
Vz=zeros(size(k0));
NormeV=sqrt(Vx.^2+Vy.^2+Vz.^2);
Vx = Vx ./NormeV;
Vy = Vy ./NormeV;
Vz = Vz ./NormeV;
Tx = Ny.*Vz - Nz.*Vy;
Ty = -Nx.*Vz + Nz.*Vx;
Tz = Nx.*Vy - Ny.*Vx;


%                  GENERATE DIRECTOTY FOR AMITEX
%-----------------------------------------------

normale = [Nx,Ny,Nz];
direction=[Vx,Vy,Vz];

FV=[fv,1-fv];		% fv = volume fraction of beam matrix
phases = [2,1];		% => volume fraction of material numM=2 - numZ=1 <=>(here : MATE ID=1 / ZONE ID=0) 
zone=ones(size(k0));
zone=uint64([zone,zone]);
position = uint64(k0);
loi ='laminate';

% gather information in a cell object for use in save_composite_voxels
Mat = Composite_Material(position,zone,loi,phases,FV,normale,direction); 
MatComp=cell(1);
MatComp{1}=Mat;

% Generate directory for amitex_fftp descriptioni of composite voxels
save_composite_voxels(MatComp,Rep_composite );


