classdef Composite_Material
%COMPOSITE_MATERIAL description of composite materials for AMITEX_FFTP
%
%
%	AMITEX_FFTP compatible description of composite voxels 
%
%	a composite material is defined by 
%          - a number of phases with their materialID's 
%	   - the homogenization Law
%          - for each composite voxel : 
%		the zoneID in the materialID
%		the volume fractions of phases
%		the interface Normal and Direction vectors (for laminate Law)
%
%===========================================================================
%
% Usage :
%--------
%	Mat = Composite_Material(position,zone,loi,phases,fv,normal,direction,surface)
%
% Properties :
%-------------
%	Position:	list of linear index of the voxels, vector (uint64(npts,1))
%	Phase   :	list of material IDs for the phases, table of integer (nphases)
%                       example : [matIDphase1,matIDphase2,matIDphase3]
%	Zone	:	list of zone IDs for the phases, table uint64(npts,nphases)
%			example : [ZoneIDPhase1, ZoneIDPhase2, ZoneIDPhase3]
%	Law	:	homogenization rule : 'laminate' or 'voigt' or 'reuss'
%	Normal, Direction : list of normal and tangent vectors, table real (npts,3)
%			orientation of the normal in the direction of increasing IDs
%	Fv	:	list of volume fractions, real(npts, nphases)
%	Surface :	OPTIONAL 
%			list of surfaces between phases (1/2, 2/3,... N-1/N), real(npts,nphases-1)
%			only consistent with laminate voxels
%
% WARNING : 	A UNIQUE (Normal/Direction) is given for the composite voxel
%		I practice, this information is usefull only for laminate composites
%
%---------------------------------------------------------------------------	
%	
% OCTAVE : 
%          
% MATLAB : 
%
%---------------------------------------------------------------------------
% AUTHOR : A. MARANO, L. GELEBART
%
% Modification :
%	17/12/2019 : first version in microgen_fft
%
%===========================================================================
properties
    Position
    Zone
    Law
    Phases
    Fv
    Normal
    Direction
    Surface
end

methods
    function Mat = Composite_Material(position,zone,law,phases,fv,normal,direction,surface)
        
        
        if ( ~strcmp(law,'laminate') && ~strcmp(law,'voigt') && ~strcmp(law,'reuss') )
            error('Law must be voigt, or reuss or laminate');
        end
        
        Uposition = unique(position);
        if (size(Uposition) ~= size(position) );
            error('position contains double values');
        end;
        
        nvc = length(position);
        if (size(zone,1) ~= nvc || size(fv,1) ~= nvc ||  size(normal,1) ~= nvc ||  size(direction,1) ~= nvc  )
            error('Size incompatible with the size of Position');
        end
        
        nph = length(phases);
        if (size(zone,2) ~= nph || size(fv,2) ~= nph)
            error('Size incompatible with the size of phases')
        end
        
        if (class(position)~="uint64" || class(zone)~="uint64")
            error(' position and zone must be of type uint64')
        end
        
        Mat.Position  = position;
        Mat.Zone      = zone;
        Mat.Law       = law;
        Mat.Phases    = phases;
        Mat.Fv        = fv;
        Mat.Normal   = normal;
        Mat.Direction = direction;
        if (nargin > 8)
           if (size(surface,1) ~= nvc )
               error('Size incompatible with the size of Position');
           end
           if (size(surface,2) ~= nph - 1 )
               error('Size incompatible with the size of phases');
           end
           Mat.Surface   = surface;
        end
    end;
end

end

